@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology  
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology  
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 #  include <harminv.h>
 #endif
 
+using namespace std;
+
 namespace meep {
 
 #define BAND(b,r,t) ((b)[(r)+(t)*nr])
@@ -75,8 +77,8 @@ void fields::prepare_for_bands(const vec &p, double endtime, double fmax,
     int indind = 0;
     while (bands->index[indind] != -1 && indind < num_bandpts) indind++;
     for (int h=0;h<num_chunks;h++)
-      if (chunks[h]->v.contains(p)) {
-        chunks[h]->v.interpolate(chunks[h]->v.eps_component(), p, ind, w);
+      if (chunks[h]->gv.contains(p)) {
+        chunks[h]->gv.interpolate(chunks[h]->gv.eps_component(), p, ind, w);
         for (int i=0;i<8&&w[i]&&indind<num_bandpts;i++) {
           bands->chunk[indind] = chunks[h];
           bands->index[indind++] = ind[i];
@@ -90,8 +92,8 @@ void fields::prepare_for_bands(const vec &p, double endtime, double fmax,
   const double epsmax = max_eps();
 
   double cutoff_freq = 0.0;
-  if (v.dim == Dcyl) {
-    cutoff_freq = 1.84*a*dt/(2*pi)/v.nr()/sqrt(epsmax);
+  if (gv.dim == Dcyl) {
+    cutoff_freq = 1.84*a*dt/(2*pi)/gv.nr()/sqrt(epsmax);
     if (m == 0) cutoff_freq *= 0.5;
   }
   bands->fmin = sqrt(cutoff_freq*cutoff_freq + abs(k[Z])*abs(k[Z])*(a*dt)*(a*dt)/epsmax); // FIXME
@@ -118,7 +120,7 @@ void fields::prepare_for_bands(const vec &p, double endtime, double fmax,
   bands->dt = dt * bands->scale_factor;
   for (int c=0;c<10;c++)
     for (int i=0;i<num_bandpts;i++)
-      if (v.has_field((component)c)) {
+      if (gv.has_field((component)c)) {
         delete[] bands->f[i][c];
         bands->f[i][c] = new complex<double>[bands->ntime];
         if (bands->f[i][c] == NULL)
@@ -146,7 +148,7 @@ void fields_chunk::record_bands(int tcount) {
   for (int p=0; p<num_bandpts && bands->index[p]!=-1; p++)
     if (this == bands->chunk[p])
       for (int c=0;c<10;c++)
-        if (v.has_field((component)c)) {
+        if (gv.has_field((component)c)) {
           complex<double> tmp;
           if (f[c][0] && f[c][1]) tmp = complex<double>(f[c][0][bands->index[p]],
                                                         f[c][1][bands->index[p]]);
@@ -349,7 +351,7 @@ complex<double> *fields::clever_cluster_bands(int maxbands, double *approx_power
 
   for (int p=0; p<num_bandpts && bands->index[p]!=-1; p++)
     for (int whichf = 0; whichf < 10; whichf++)
-      if (v.has_field((component)whichf) && maxbands < max_freqs - freqs_so_far) {
+      if (gv.has_field((component)whichf) && maxbands < max_freqs - freqs_so_far) {
         if (verbosity>1) master_printf("Looking at field %d\n", whichf);
         int freqs_here = bands->get_freqs(bands->f[p][whichf], ntime,
                                           ta+freqs_so_far,
@@ -361,7 +363,7 @@ complex<double> *fields::clever_cluster_bands(int maxbands, double *approx_power
         }
         if (freqs_so_far + maxbands > max_freqs) break;
       }
-  if (k == 0 && v.dim == Dcyl && m != 0) fields_considered /= 2;
+  if (k == 0 && gv.dim == Dcyl && m != 0) fields_considered /= 2;
   num_found = cluster_some_bands_cleverly(tf, td, ta, freqs_so_far, fields_considered,
                                           maxbands, fad, approx_power);
   delete[] ta;
@@ -400,6 +402,12 @@ complex<double> *fields::clever_cluster_bands(int maxbands, double *approx_power
   return fad;
 }
 
+/* backwards compatibility with harminv < 1.4 */
+#if HARMINV_VERSION_MAJOR < 1 || (HARMINV_VERSION_MAJOR == 1 && HARMINV_VERSION_MINOR < 4)
+#  define harminv_get_amplitude(pa, d, k) *(pa) = harminv_get_amplitude(d, k)
+#  define harminv_get_omega(pw, d, k) *(pw) = harminv_get_omega(d, k)
+#endif
+
 int bandsdata::get_freqs(complex<double> *data, int n, complex<double> *amps,
                          double *freq_re, double *freq_im) {
   
@@ -466,9 +474,12 @@ int do_harminv(complex<double> *data, int n, double dt,
       }
   
   double min_err = harminv_get_freq_error(hd, fsort[0]);
-  double max_amp = abs(harminv_get_amplitude(hd, 0));
+  complex<double> aa;
+  harminv_get_amplitude(&aa, hd, 0);
+  double max_amp = abs(aa);
   for (int i = 1; i < nf; ++i) {
-    double amp = abs(harminv_get_amplitude(hd, i));
+    harminv_get_amplitude(&aa, hd, i);
+    double amp = abs(aa);
     if (max_amp < amp)
       max_amp = amp;
   }
@@ -477,7 +488,8 @@ int do_harminv(complex<double> *data, int n, double dt,
     for (int i = 0; i < nf; ++i) {
       double f = abs(harminv_get_freq(hd, fsort[i]) / dt);
       double err = harminv_get_freq_error(hd, fsort[i]);
-      double amp = abs(harminv_get_amplitude(hd, fsort[i]));
+      harminv_get_amplitude(&aa, hd, fsort[i]);
+      double amp = abs(aa);
       if (f >= fmin && f <= fmax
 	  && abs(harminv_get_Q(hd, fsort[i])) > Q_thresh
 	  && err < err_thresh
@@ -536,10 +548,12 @@ int do_harminv(complex<double> *data, int n, double dt,
       }
   
   for (int i = 0; i < nf; ++i) {
-    complex<double> freq = harminv_get_omega(hd, fsort[i]) / (2*pi*dt);
+    complex<double> freq;
+    harminv_get_omega(&freq, hd, fsort[i]);
+    freq /= (2*pi*dt);
     freq_re[i] = abs(real(freq));
     freq_im[i] = imag(freq);
-    amps[i] = harminv_get_amplitude(hd, fsort[i]);
+    harminv_get_amplitude(&(amps[i]), hd, fsort[i]);
     if (errors)
       errors[i] = harminv_get_freq_error(hd, fsort[i]);
   }
