@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 
 /* Below are the monitor point routines. */
 
+using namespace std;
+
 namespace meep {
 
 monitor_point::monitor_point() {
@@ -44,7 +46,7 @@ monitor_point::~monitor_point() {
   if (next) delete next;
 }
 
-inline complex<double> getcm(const double * const f[2], int i) {
+inline complex<double> getcm(const realnum * const f[2], int i) {
   return complex<double>(f[0][i],f[1][i]);
 }
 
@@ -74,7 +76,7 @@ void fields::get_point(monitor_point *pt, const vec &loc) const {
   pt->loc = loc;
   pt->t = time();
   FOR_COMPONENTS(c)
-    if (v.has_field(c))
+    if (gv.has_field(c))
       pt->f[c] = get_field(c,loc);
 }
 
@@ -84,7 +86,7 @@ complex<double> fields::get_field(int c, const vec &loc) const {
 }
 
 double fields::get_field(derived_component c, const vec &loc) const {
-  component c1, c2;
+  component c1 = Ex, c2 = Ex;
   double sum = 0;
   switch (c) {
   case Sx: case Sy: case Sz: case Sr: case Sp:
@@ -101,12 +103,12 @@ double fields::get_field(derived_component c, const vec &loc) const {
     return sum;
   case EnergyDensity: case D_EnergyDensity: case H_EnergyDensity:
     if (c != H_EnergyDensity)
-      FOR_ELECTRIC_COMPONENTS(c1) if (v.has_field(c1)) {
+      FOR_ELECTRIC_COMPONENTS(c1) if (gv.has_field(c1)) {
 	c2 = direction_component(Dx, component_direction(c1));
 	sum += real(conj(get_field(c1, loc)) * get_field(c2, loc));
       }
     if (c != D_EnergyDensity)
-      FOR_MAGNETIC_COMPONENTS(c1) if (v.has_field(c1)) {
+      FOR_MAGNETIC_COMPONENTS(c1) if (gv.has_field(c1)) {
 	complex<double> f = get_field(c1, loc);
 	sum += real(conj(f) * f);
       }
@@ -120,13 +122,13 @@ complex<double> fields::get_field(component c, const vec &loc) const {
   case Dielectric:
     return get_eps(loc);
   case Permeability:
-    abort("get_field(Permeability) unimplemented");
+    return get_mu(loc);
   default:
     ivec ilocs[8];
     double w[8];
     complex<double> val[8];
     for (int i=0;i<8;i++) val[i] = 0.0;
-    v.interpolate(c, loc, ilocs, w);
+    gv.interpolate(c, loc, ilocs, w);
     for (int argh=0;argh<8&&w[argh];argh++)
       val[argh] = w[argh]*get_field(c,ilocs[argh]);
     dumbsort(val);
@@ -142,7 +144,7 @@ complex<double> fields::get_field(component c, const ivec &origloc) const {
   locate_point_in_user_volume(&iloc, &kphase);
   for (int sn=0;sn<S.multiplicity();sn++)
     for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->v.contains(S.transform(iloc,sn)))
+      if (chunks[i]->gv.contains(S.transform(iloc,sn)))
         return S.phase_shift(c,sn)*kphase*
           chunks[i]->get_field(S.transform(c,sn),S.transform(iloc,sn));
   return 0.0;
@@ -150,20 +152,20 @@ complex<double> fields::get_field(component c, const ivec &origloc) const {
 
 complex<double> fields_chunk::get_field(component c, const ivec &iloc) const {
   complex<double> res = 0.0;
-  if (f[c][0] && f[c][1]) res = getcm(f[c], v.index(c, iloc));
-  else if (f[c][0]) res = f[c][0][v.index(c,iloc)];
+  if (f[c][0] && f[c][1]) res = getcm(f[c], gv.index(c, iloc));
+  else if (f[c][0]) res = f[c][0][gv.index(c,iloc)];
   return broadcast(n_proc(), res);
 }
 
 /* Bounding box for zero-communication get_field, below.  This is the
    largest box in which you can interpolate the fields without communication.
    It is *not* necessarily non-overlapping with other chunks. */
-geometric_volume fields_chunk::get_field_gv(component c) const {
+volume fields_chunk::get_field_gv(component c) const {
   switch (c) {
   case Dielectric: case Permeability:
-    c = v.eps_component();
+    c = gv.eps_component();
   default:
-    return geometric_volume(v.loc(c, 0), v.loc(c, v.ntot() - 1));
+    return volume(gv.loc(c, 0), gv.loc(c, gv.ntot() - 1));
   }
 }
 
@@ -174,165 +176,31 @@ complex<double> fields_chunk::get_field(component c, const vec &loc) const {
   double w[8];
   switch (c) {
   case Permeability: abort("non-collective get_field(mu) unimplemented");
-  case Dielectric: {
-    c = v.eps_component();
-    v.interpolate(c, loc, ilocs, w);
-    double res = 0.0;
-    for (int i = 0; i < 8 && w[i] != 0.0; ++i) {
-      if (!v.contains(ilocs[i]))
-	abort("invalid loc in chunk get_field, weight = %g", w[i]);
-      res += s->eps[v.index(c,ilocs[i])] * w[i];
-    }
-    return res;
-  }
+  case Dielectric: abort("non-collective get_field(eps) unimplemented");
   default: {
-    v.interpolate(c, loc, ilocs, w);
+    gv.interpolate(c, loc, ilocs, w);
     complex<double> res = 0.0;
     for (int i = 0; i < 8 && w[i] != 0.0; ++i) {
-      if (!v.contains(ilocs[i]))
+      if (!gv.contains(ilocs[i]))
 	abort("invalid loc in chunk get_field, weight = %g", w[i]);
-      if (f[c][0] && f[c][1]) res += getcm(f[c], v.index(c, ilocs[i])) * w[i];
-      else if (f[c][0]) res += f[c][0][v.index(c,ilocs[i])] * w[i];
+      if (f[c][0] && f[c][1]) res += getcm(f[c], gv.index(c, ilocs[i])) * w[i];
+      else if (f[c][0]) res += f[c][0][gv.index(c,ilocs[i])] * w[i];
     }
     return res;
   }
   }
 }
 
-complex<double> fields_chunk::get_polarization_field(const polarizability_identifier &p,
-                                                     component c, const ivec &iloc) const {
-  complex<double> res = 0.0;
-  for (polarization *P = olpol; P; P = P->next)
-    if (P->pb->get_identifier() == p) {
-      if (P->P[c][0] && P->P[c][1]) res = getcm(P->P[c], v.index(c, iloc));
-      else if (P->P[c][0]) res = P->P[c][0][v.index(c,iloc)];
-    }
-  return broadcast(n_proc(), res);
-}
-
-double fields::get_polarization_energy(const vec &loc) const {
-  ivec ilocs[8];
-  double w[8], val[8];
-  for (int i=0;i<8;i++) val[i] = 0.0;
-  v.interpolate(v.eps_component(), loc, ilocs, w);
-  for (int argh=0;argh<8&&w[argh];argh++)
-    val[argh] = w[argh]*get_polarization_energy(ilocs[argh]);
-  dumbsort(val);
-  double res = 0.0;
-  for (int i=0;i<8;i++) res += val[i];
-  return res;
-}
-
-double fields::get_polarization_energy(const ivec &origloc) const {
-  ivec iloc = origloc;
-  complex<double> aaack = 1.0;
-  locate_point_in_user_volume(&iloc, &aaack);
-  for (int sn=0;sn<S.multiplicity();sn++)
-    for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->v.contains(S.transform(iloc,sn)))
-        return chunks[i]->get_polarization_energy(S.transform(iloc,sn));
-  return 0.0;
-}
-
-double fields_chunk::get_polarization_energy(const ivec &iloc) const {
-  double res = 0.0;
-  polarization *p = pol;
-  while (is_mine() && p) {
-    res += p->local_energy(iloc);
-    p = p->next;
-  }
-  return broadcast(n_proc(), res);
-}
-
-double fields_chunk::my_polarization_energy(const ivec &iloc) const {
-  if (!is_mine())
-    abort("Can't call my_polarization_energy on someone else's chunk!\n");
-  double res = 0.0;
-  polarization *p = pol;
-  while (p) {
-    res += p->local_energy(iloc);
-    p = p->next;
-  }
-  return res;
-}
-
-double fields::get_polarization_energy(const polarizability_identifier &p,
-                                       const vec &loc) const {
-  ivec ilocs[8];
-  double w[8], val[8];
-  for (int i=0;i<8;i++) val[i] = 0.0;
-  v.interpolate(v.eps_component(), loc, ilocs, w);
-  for (int argh=0;argh<8&&w[argh];argh++)
-    val[argh] = w[argh]*get_polarization_energy(p, ilocs[argh]);
-  dumbsort(val);
-  double res = 0.0;
-  for (int i=0;i<8;i++) res += val[i];
-  return res;
-}
-
-double fields::get_polarization_energy(const polarizability_identifier &p,
-                                       const ivec &origloc) const {
-  ivec iloc = origloc;
-  complex<double> aaack = 1.0;
-  locate_point_in_user_volume(&iloc, &aaack);
-  for (int sn=0;sn<S.multiplicity();sn++)
-    for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->v.contains(S.transform(iloc,sn)))
-        return chunks[i]->get_polarization_energy(p, S.transform(iloc,sn));
-  return 0.0;
-}
-
-double fields_chunk::get_polarization_energy(const polarizability_identifier &pi,
-                                             const ivec &iloc) const {
-  double res = 0.0;
-  polarization *p = pol;
-  while (is_mine() && p) {
-    if (p->pb->get_identifier() == pi) res += p->local_energy(iloc);
-    p = p->next;
-  }
-  return broadcast(n_proc(), res);
-}
-
-double fields_chunk::my_polarization_energy(const polarizability_identifier &pi,
-                                            const ivec &iloc) const {
-  if (!is_mine())
-    abort("Can't call my_polarization_energy on someone else's chunk!\n");
-  double res = 0.0;
-  polarization *p = pol;
-  while (p) {
-    if (p->pb->get_identifier() == pi) res += p->local_energy(iloc);
-    p = p->next;
-  }
-  return res;
-}
-
-double fields::get_eps(const ivec &origloc) const {
-  ivec iloc = origloc;
-  complex<double> aaack = 1.0;
-  locate_point_in_user_volume(&iloc, &aaack);
-  for (int sn=0;sn<S.multiplicity();sn++)
-    for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->v.contains(S.transform(iloc,sn)))
-        return chunks[i]->get_eps(S.transform(iloc,sn));
-  return 0.0;
-}
-
-double fields_chunk::get_eps(const ivec &iloc) const {
-  double res = 0.0;
-  if (is_mine()) res = s->eps[v.index(v.eps_component(), iloc)];
-  return broadcast(n_proc(), res);
-}
-
-double fields::get_inveps(component c, direction d,
+double fields::get_chi1inv(component c, direction d,
 			  const ivec &origloc) const {
   ivec iloc = origloc;
   complex<double> aaack = 1.0;
   locate_point_in_user_volume(&iloc, &aaack);
   for (int sn=0;sn<S.multiplicity();sn++)
     for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->v.contains(S.transform(iloc,sn))) {
+      if (chunks[i]->gv.contains(S.transform(iloc,sn))) {
 	signed_direction ds = S.transform(d,sn);
-        return chunks[i]->get_inveps(S.transform(c,sn), ds.d,
+        return chunks[i]->get_chi1inv(S.transform(c,sn), ds.d,
 				     S.transform(iloc,sn))
 	     * (ds.flipped ^ S.transform(component_direction(c),sn).flipped
 		? -1 : 1);
@@ -340,22 +208,23 @@ double fields::get_inveps(component c, direction d,
   return 0.0;
 }
 
-double fields_chunk::get_inveps(component c, direction d,
+double fields_chunk::get_chi1inv(component c, direction d,
 			     const ivec &iloc) const {
   double res = 0.0;
-  if (is_mine()) res = s->inveps[c][d] ? s->inveps[c][d][v.index(c, iloc)] : 0;
+  if (is_mine()) res = s->chi1inv[c][d] ? s->chi1inv[c][d][gv.index(c, iloc)]
+		   : (d == component_direction(c) ? 1.0 : 0);
   return broadcast(n_proc(), res);
 }
 
-double fields::get_inveps(component c, direction d,
+double fields::get_chi1inv(component c, direction d,
 			  const vec &loc) const {
   ivec ilocs[8];
   double w[8];
   double val[8];
   for (int i=0;i<8;i++) val[i] = 0.0;
-  v.interpolate(c, loc, ilocs, w);
+  gv.interpolate(c, loc, ilocs, w);
   for (int argh=0;argh<8&&w[argh];argh++)
-    val[argh] = w[argh]*get_inveps(c,d,ilocs[argh]);
+    val[argh] = w[argh]*get_chi1inv(c,d,ilocs[argh]);
   dumbsort(val);
   double res = 0.0;
   for (int i=0;i<8;i++) res += val[i];
@@ -366,44 +235,56 @@ double fields::get_eps(const vec &loc) const {
   double tr = 0;
   int nc = 0;
   FOR_ELECTRIC_COMPONENTS(c)
-    if (v.has_field(c)) {
-      tr += get_inveps(c, component_direction(c), loc);
+    if (gv.has_field(c)) {
+      tr += get_chi1inv(c, component_direction(c), loc);
       ++nc;
     }
   return nc / tr;
 }
 
-double structure::get_inveps(component c, direction d,
+double fields::get_mu(const vec &loc) const {
+  double tr = 0;
+  int nc = 0;
+  FOR_MAGNETIC_COMPONENTS(c)
+    if (gv.has_field(c)) {
+      tr += get_chi1inv(c, component_direction(c), loc);
+      ++nc;
+    }
+  return nc / tr;
+}
+
+double structure::get_chi1inv(component c, direction d,
 			     const ivec &origloc) const {
   ivec iloc = origloc;
   for (int sn=0;sn<S.multiplicity();sn++)
     for (int i=0;i<num_chunks;i++)
-      if (chunks[i]->v.contains(S.transform(iloc,sn))) {
+      if (chunks[i]->gv.contains(S.transform(iloc,sn))) {
 	signed_direction ds = S.transform(d,sn);
-        return chunks[i]->get_inveps(S.transform(c,sn), ds.d,
-				     S.transform(iloc,sn))
-	     * (ds.flipped ^ S.transform(component_direction(c),sn).flipped
+        return chunks[i]->get_chi1inv(S.transform(c,sn), ds.d,
+				      S.transform(iloc,sn))
+	  * (ds.flipped ^ S.transform(component_direction(c),sn).flipped
 		? -1 : 1);
       }
   return 0.0;
 }
 
-double structure_chunk::get_inveps(component c, direction d,
-				   const ivec &iloc) const {
+double structure_chunk::get_chi1inv(component c, direction d,
+				    const ivec &iloc) const {
   double res = 0.0;
-  if (is_mine()) res = inveps[c][d] ? inveps[c][d][v.index(c, iloc)] : 0;
+  if (is_mine()) res = chi1inv[c][d] ? chi1inv[c][d][gv.index(c, iloc)] 
+		   : (d == component_direction(c) ? 1.0 : 0);
   return broadcast(n_proc(), res);
 }
 
-double structure::get_inveps(component c, direction d,
-			     const vec &loc) const {
+double structure::get_chi1inv(component c, direction d,
+			      const vec &loc) const {
   ivec ilocs[8];
   double w[8];
   double val[8];
   for (int i=0;i<8;i++) val[i] = 0.0;
-  v.interpolate(c, loc, ilocs, w);
+  gv.interpolate(c, loc, ilocs, w);
   for (int argh=0;argh<8&&w[argh];argh++)
-    val[argh] = w[argh]*get_inveps(c,d,ilocs[argh]);
+    val[argh] = w[argh]*get_chi1inv(c,d,ilocs[argh]);
   dumbsort(val);
   double res = 0.0;
   for (int i=0;i<8;i++) res += val[i];
@@ -414,8 +295,19 @@ double structure::get_eps(const vec &loc) const {
   double tr = 0;
   int nc = 0;
   FOR_ELECTRIC_COMPONENTS(c)
-    if (v.has_field(c)) {
-      tr += get_inveps(c, component_direction(c), loc);
+    if (gv.has_field(c)) {
+      tr += get_chi1inv(c, component_direction(c), loc);
+      ++nc;
+    }
+  return nc / tr;
+}
+
+double structure::get_mu(const vec &loc) const {
+  double tr = 0;
+  int nc = 0;
+  FOR_MAGNETIC_COMPONENTS(c)
+    if (gv.has_field(c)) {
+      tr += get_chi1inv(c, component_direction(c), loc);
       ++nc;
     }
   return nc / tr;
@@ -445,13 +337,13 @@ double monitor_point::poynting_in_direction(direction d) {
   return (real(E1)*real(H2) - real(E2)*real(H1)) + (imag(E1)*imag(H2) - imag(E2)*imag(H1));
 }
 
-double monitor_point::poynting_in_direction(vec v) {
-  if (v.dim != loc.dim)
-    abort("poynting_in_direction: v.dim != loc.dim\n");
-  v = v / abs(v);
+double monitor_point::poynting_in_direction(vec dir) {
+  if (dir.dim != loc.dim)
+    abort("poynting_in_direction: dir.dim != loc.dim\n");
+  dir = dir / abs(dir);
   double result = 0.0;
-  LOOP_OVER_DIRECTIONS(v.dim, d)
-    result += v.in_direction(d) * poynting_in_direction(d);
+  LOOP_OVER_DIRECTIONS(dir.dim, d)
+    result += dir.in_direction(d) * poynting_in_direction(d);
   return result;
 }
 
