@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology  
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology  
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -20,19 +20,16 @@
 
 #include <meep.hpp>
 using namespace meep;
+using namespace std;
 
 double one(const vec &) { return 1.0; }
 
-double slab(const vec &v) {
-  abort("not yet\n");
-}
-
 int radiating_2D(const double xmax) {
   const double a = 10.0;
   const double ymax = 3.0;
 
-  volume v = voltwo(xmax,ymax,a);
-  structure s(v, one, pml(ymax/3));
+  grid_volume gv = voltwo(xmax,ymax,a);
+  structure s(gv, one, pml(ymax/3));
 
   fields f(&s);
   double w = 0.30;
@@ -66,9 +63,9 @@ int radiating_3D(const double xmax) {
   const double a = 10.0;
   const double ymax = 3.0;
 
-  volume v = vol3d(xmax,ymax,ymax,a);
-  symmetry S = mirror(Y,v) - mirror(Z,v);
-  structure s(v, one, pml(ymax/3));
+  grid_volume gv = vol3d(xmax,ymax,ymax,a);
+  symmetry S = mirror(Y,gv) - mirror(Z,gv);
+  structure s(gv, one, pml(ymax/3));
 
   fields f(&s);
   double w = 0.30;
